@@ -21,6 +21,7 @@
 #'         If \code{B} is present, returns the reduced row-echelon form of \code{A}, with the
 #'         same operations applied to \code{B}.
 #' @author John Fox
+#' @export
 #' @examples
 #'   A <- matrix(c(2, 1, -1,
 #'                -3, -1, 2,
@@ -46,25 +47,18 @@ gaussianElimination <- function(A, B, tol=sqrt(.Machine$double.eps),
     # fractions: try to express nonintegers as rational numbers
     # If B is absent returns the reduced row-echelon form of A.
     # If B is present, reduces A to RREF carrying B along.
-    printMatrix <- function(A){
-        if (latex) {
-            matrix2latex(A, fractions=fractions, digits = round(abs(log(tol,10))))
-        }
-        else {
-            if (fractions) print(fraction(as.matrix(A)))
-            else print(round(as.matrix(A), round(abs(log(tol,10)))))
-        }
-    }
-    if (fractions) {
-        mass <- requireNamespace("MASS", quietly=TRUE)
-        if (!mass) stop("fractions=TRUE needs MASS package")
-        fraction <- MASS::fractions
-        frac <- function(x) as.character(fraction(x))
-    }
     if ((!is.matrix(A)) || (!is.numeric(A)))
         stop("argument must be a numeric matrix")
     n <- nrow(A)
     m <- ncol(A)
+    det <- 1
+    pivots <- rep(0, n)
+    interchanges <- 0
+    reduced <- TRUE
+    if(!is.null(attr(A, 'reduced'))){ # only used from echelon()
+    	reduced <- attr(A, 'reduced')
+    	attr(A, 'reduced') <- NULL
+    }
     if (!missing(B)){
         B <- as.matrix(B)
         if (!(nrow(B) == nrow(A)) || !is.numeric(B))
@@ -84,12 +78,16 @@ gaussianElimination <- function(A, B, tol=sqrt(.Machine$double.eps),
             # find maximum pivot in current column at or below current row
             which <- which.max(abs(currentColumn))
             pivot <- currentColumn[which]
+            det <- det*pivot
+            pivots[i] <- pivot
             if (abs(pivot) <= tol) { # check for 0 pivot
                 j <- j + 1
                 next
             }
             if (which > i) {
                 A <- rowswap(A, i, which) # exchange rows (E3)
+                det <- -det
+                interchanges <- interchanges + 1
                 if (verbose) {
                     cat("\n exchange rows", i, "and", which, "\n")
                     printMatrix(A)
@@ -97,19 +95,20 @@ gaussianElimination <- function(A, B, tol=sqrt(.Machine$double.eps),
             }
             A <- rowmult(A, i, 1/pivot) # pivot (E1)
             if (verbose && abs(pivot - 1) > tol){
-                cat("\n multiply row", i, "by", 
-                    if (fractions) frac(1/pivot) else 1/pivot, "\n")
+                cat("\n multiply row", i, "by",
+                    if (fractions) as.character(MASS::fractions(1/pivot)) else 1/pivot, "\n")
                 printMatrix(A)
             }
             for (k in 1:n){
                 if (k == i) next
+            	if (!reduced && k < j) next
                 factor <- A[k, j]
                 if (abs(factor) < tol) next
                 A <- rowadd(A, i, k, -factor) # sweep column j (E2)
                 if (verbose){
                   if (abs(factor - 1) > tol){
                     cat("\n multiply row", i, "by",
-                        if (fractions) frac(abs(factor)) else abs(factor),
+                        if (fractions) as.character(MASS::fractions(abs(factor))) else abs(factor),
                         if (factor > 0) "and subtract from row" else "and add to row", k, "\n")
                   }
                   else{
@@ -132,10 +131,32 @@ gaussianElimination <- function(A, B, tol=sqrt(.Machine$double.eps),
         A <- rbind(A, zeroRows)
     }
     rownames(A) <- NULL
-    ret <- if (fractions) fraction(A) else round(A, round(abs(log(tol, 10))))
+    ret <- formatNumbers(A, fractions=fractions, tol=tol)
+    if (m == n) {
+        attr(ret, "det") <- det
+        attr(ret, "pivots") <- pivots
+        attr(ret, "interchanges") <- interchanges
+        class(ret) <- c("enhancedMatrix", "matrix")
+    }
     if (verbose) invisible(ret) else ret
 }
 
+#' Print method for enhancedMatrix objects
+#'
+#' @param x matrix to print
+#' @param ... arguments to pass down
+#' @rdname gaussianElimination
+#' @export
+#'
+print.enhancedMatrix <- function(x, ...){
+    attr(x, "det") <- NULL
+    attr(x, "pivots") <- NULL
+    attr(x, "interchanges") <- NULL
+    attr(x, "T") <- NULL
+    class(x) <- "matrix"
+    print(x, ...)
+}
+
 #' Inverse of a Matrix
 #'
 #' Uses \code{\link{gaussianElimination}} to find the inverse of a square, non-singular matrix, \eqn{X}.
@@ -150,6 +171,7 @@ gaussianElimination <- function(A, B, tol=sqrt(.Machine$double.eps),
 #' @param ... other arguments passed on
 #' @return the inverse of \code{X}
 #' @author John Fox
+#' @export
 #' @examples
 #'   A <- matrix(c(2, 1, -1,
 #'                -3, -1, 2,
@@ -169,6 +191,7 @@ Inverse <- function(X, tol=sqrt(.Machine$double.eps), ...){
     X[,(n + 1):(2*n)]  # return inverse
 }
 # synonym
+#' @export
 inv <- function(X, ...) Inverse(X, tol=sqrt(.Machine$double.eps), ...)
 
 #RREF <- function(X, ...) gaussianElimination(X, ...)
@@ -177,26 +200,33 @@ inv <- function(X, ...) Inverse(X, tol=sqrt(.Machine$double.eps), ...)
 
 #' Echelon Form of a Matrix
 #'
-#' Returns the reduced row-echelon form of the matrix \code{X}, using \code{\link{gaussianElimination}}.
-#' It is nothing more than a synonym for Gaussian elimination, but offers the possibility to show the
-#' steps using \code{verbose=TRUE}.
+#' Returns the (reduced) row-echelon form of the matrix \code{A}, using \code{\link{gaussianElimination}}.
 #'
-#' When the matrix \code{x} is square and non-singular, the result will be the identity matrix. Otherwise,
-#' the result will have some all-zero rows, and the rank of the matrix is the number of not all-zero rows.
+#' When the matrix \code{A} is square and non-singular, the reduced row-echelon result will be the 
+#' identity matrix, while the row-echelon from will be an upper triagle matrix. 
+#' Otherwise, the result will have some all-zero rows, and the rank of the matrix 
+#' is the number of not all-zero rows.
 #'
-#' @param X a matrix
+#' @param A coefficient matrix
+#' @param B right-hand side vector or matrix. If \code{B} is a matrix, the result gives solutions for each column as the right-hand
+#'        side of the equations with coefficients in \code{A}.
+#' @param reduced logical; should reduced row echelon form be returned? If \code{FALSE} a non-reduced 
+#'   row echelon form will be returned
 #' @param ... other arguments passed to \code{gaussianElimination}
 #' @return the reduced echelon form of \code{X}.
 #' @author John Fox
+#' @export
 #' @examples
 #' A <- matrix(c(2, 1, -1,
 #'              -3, -1, 2,
 #'              -2,  1, 2), 3, 3, byrow=TRUE)
 #' b <- c(8, -11, -3)
-#' echelon(A, b, verbose=TRUE, fractions=TRUE)
+#' echelon(A, b, verbose=TRUE, fractions=TRUE) # reduced row-echelon form
+#' echelon(A, b, reduced=FALSE, verbose=TRUE, fractions=TRUE) # row-echelon form
 #'
 #' A <- matrix(c(1,2,3,4,5,6,7,8,10), 3, 3) # a nonsingular matrix
 #' A
+#' echelon(A, reduced=FALSE) # the row-echelon form of A
 #' echelon(A) # the reduced row-echelon form of A
 #'
 #' b <- 1:3
@@ -206,11 +236,15 @@ inv <- function(X, ...) Inverse(X, tol=sqrt(.Machine$double.eps), ...)
 #' B <- matrix(1:9, 3, 3) # a singular matrix
 #' B
 #' echelon(B)
+#' echelon(B, reduced=FALSE)
 #' echelon(B, b)
 #' echelon(B, diag(3))
 #'
 
-echelon <- function(X, ...) gaussianElimination(X, ...)
+echelon <- function(A, B, reduced = TRUE, ...) {
+	attr(A, "reduced") <- reduced
+	gaussianElimination(A, B, ...)
+}
 
 #' Generalized Inverse of a Matrix
 #'
@@ -227,6 +261,7 @@ echelon <- function(X, ...) gaussianElimination(X, ...)
 #' @param fractions logical; if \code{TRUE}, try to express non-integers as rational numbers
 #' @return the generalized inverse of \code{A}, expressed as fractions if \code{fractions=TRUE}, or rounded
 #' @seealso \code{\link[MASS]{ginv}} for a more generally usable function
+#' @export
 #' @author John Fox
 #'
 #' @examples
@@ -249,10 +284,6 @@ Ginv <- function(A, tol=sqrt(.Machine$double.eps), verbose=FALSE,
     # tol: tolerance for checking for 0 pivot
     # verbose: if TRUE, print intermediate steps
     # fractions: try to express nonintegers as rational numbers
-    if (fractions) {
-        mass <- requireNamespace("MASS", quietly=TRUE)
-        if (!mass) stop("fractions=TRUE needs MASS package")
-    }
     m <- nrow(A)
     n <- ncol(A)
     B <- gaussianElimination(A, diag(m), tol=tol, verbose=verbose,
@@ -264,7 +295,7 @@ Ginv <- function(A, tol=sqrt(.Machine$double.eps), verbose=FALSE,
     R <- t(C[,-(1:m)])
     AC <- t(C[,1:m])
     ginv <- R %*% t(AC) %*% L
-    if (fractions) MASS::fractions (ginv) else round(ginv, round(abs(log(tol, 10))))
+    formatNumbers(ginv, fractions=fractions, tol=tol)
 }
 
 #' Cholesky Square Root of a Matrix
@@ -279,6 +310,7 @@ Ginv <- function(A, tol=sqrt(.Machine$double.eps), verbose=FALSE,
 #' @seealso \code{\link[base]{chol}} for the base R function
 #' @seealso \code{\link{gsorth}} for Gram-Schmidt orthogonalization of a data matrix
 #' @author John Fox
+#' @export
 #' @examples
 #' C <- matrix(c(1,2,3,2,5,6,3,6,10), 3, 3) # nonsingular, symmetric
 #' C
